package bytetype

import (
	"gitlab.com/evatix-go/core/corecomparator"
	"gitlab.com/evatix-go/core/msgtype"
)

type Variant byte

func GetSet(
	isCondition bool,
	trueValue Variant,
	falseValue Variant,
) Variant {
	if isCondition {
		return trueValue
	}

	return falseValue
}

func GetSetVariant(
	isCondition bool,
	trueValue byte,
	falseValue byte,
) Variant {
	if isCondition {
		return Variant(trueValue)
	}

	return Variant(falseValue)
}

func (v Variant) Value() byte {
	return byte(v)
}

func (v Variant) StringValue() string {
	return string(v)
}

// v + n
func (v Variant) Add(n byte) Variant {
	return Variant(v.Value() + n)
}

// v - n
func (v Variant) Subtract(n byte) Variant {
	return Variant(v.Value() - n)
}

func (v Variant) Is(n Variant) bool {
	return v.Value() == n.Value()
}

// val >= start &&  val <= end
func (v Variant) IsBetween(start, end byte) bool {
	val := v.Value()

	return val >= start && val <= end
}

// val >= start &&  val <= end
func (v Variant) IsBetweenInt(start, end int) bool {
	val := v.Value()

	return val >= byte(start) && val <= byte(end)
}

func (v Variant) IsEqual(n byte) bool {
	return v.Value() == n
}

// v.Value() > n
func (v Variant) IsGreater(n byte) bool {
	return v.Value() > n
}

// v.Value() >= n
func (v Variant) IsGreaterEqual(n byte) bool {
	return v.Value() >= n
}

// v.Value() < n
func (v Variant) IsLess(n byte) bool {
	return v.Value() < n
}

// v.Value() <= n
func (v Variant) IsLessEqual(n byte) bool {
	return v.Value() <= n
}

func (v Variant) IsEqualInt(n int) bool {
	return v.Value() == byte(n)
}

// v.Value() > n
func (v Variant) IsGreaterInt(n int) bool {
	return v.Value() > byte(n)
}

// v.Value() >= n
func (v Variant) IsGreaterEqualInt(n int) bool {
	return v.Value() >= byte(n)
}

// v.Value() < n
func (v Variant) IsLessInt(n int) bool {
	return v.Value() < byte(n)
}

// v.Value() <= n
func (v Variant) IsLessEqualInt(n int) bool {
	return v.Value() <= byte(n)
}

// Here left is v, and right is `n`
func (v Variant) IsTrue(n byte, compare corecomparator.Compare) bool {
	switch compare {
	case corecomparator.Equal:
		return v.IsEqual(n)
	case corecomparator.LeftGreater:
		return v.IsGreater(n)
	case corecomparator.LeftGreaterEqual:
		return v.IsGreaterEqual(n)
	case corecomparator.LeftLess:
		return v.IsLess(n)
	case corecomparator.LeftLessEqual:
		return v.IsLessEqual(n)
	default:
		msg := msgtype.RangeNotMeet(
			msgtype.ComparatorShouldBeWithinRanghe.String(),
			corecomparator.Min(),
			corecomparator.Max(),
			compare.Ranges())

		panic(msg)
	}
}
